@@ -1,10 +1,13 @@
 #if 0
 
+N_("Aviation, making stuff, in Finland.")
+N_("Erusan is an American who has lived all over the country, loves GNOME, and enjoys cooking — primarily American, Asian, and Indian cuisine.")
 N_("I cook a lot, particularly Italian and Indian.")
 N_("The nordic cuisine maestro.")
 N_("Ray is a Desktop Software Engineer at Red Hat and vegetarian who occasionally enjoys cooking.")
 N_("My name is Fredrik Fyksen. I am from Norway, but live in Denmark. I like food, computers and music. You can read more about me at <a href=\"https://www.fyksen.me/\">fyksen.me</a>.")
 N_("Matthias is a manager in the Desktop team at Red Hat. He likes to cook as much as he likes to code.")
+N_("Matthew is a vegan and a Computer Science student who loves FOSS.")
 
 
 #endif
