@@ -26,7 +26,7 @@
 
 #include "gr-app.h"
 #include "gr-window.h"
-#include "gr-preferences.h"
+#include "gr-chef-dialog.h"
 #include "gr-recipe-store.h"
 #include "gr-cuisine.h"
 #include "gr-shell-search-provider.h"
@@ -68,16 +68,30 @@ gr_app_activate (GApplication *app)
 }
 
 static void
-preferences_activated (GSimpleAction *action,
-                       GVariant      *parameter,
-                       gpointer       app)
+chef_information_activated (GSimpleAction *action,
+                            GVariant      *parameter,
+                            gpointer       app)
 {
-        GrPreferences *prefs;
+        GrRecipeStore *store;
+        const char *id;
+        g_autoptr(GrChef) chef = NULL;
+        GrChefDialog *dialog;
         GtkWindow *win;
 
+        store = gr_app_get_recipe_store (GR_APP (g_application_get_default ()));
+
+        id = gr_recipe_store_get_user_key (store);
+
+        if (id != NULL && id[0] != '\0')
+                chef = gr_recipe_store_get_chef (store, id);
+        else
+                chef = gr_chef_new ();
+
         win = gtk_application_get_active_window (GTK_APPLICATION (app));
-        prefs = gr_preferences_new (win);
-        gtk_window_present (GTK_WINDOW (prefs));
+        dialog = gr_chef_dialog_new (win, chef);
+        gr_chef_dialog_can_create (dialog, FALSE);
+        gtk_window_set_title (GTK_WINDOW (dialog), _("My Chef Information"));
+        gtk_window_present (GTK_WINDOW (dialog));
 }
 
 static GtkWidget *
@@ -114,61 +128,6 @@ builder_info (GtkButton *button, GtkWidget *about)
                 g_warning ("Unable to show '%s': %s", uri, error->message);
 }
 
-static void
-pixbuf_fill_rgb (GdkPixbuf *pixbuf,
-                 guint      r,
-                 guint      g,
-                 guint      b)
-{
-        guchar *pixels;
-        guchar *p;
-        guint w, h;
-
-        pixels = gdk_pixbuf_get_pixels (pixbuf);
-        h = gdk_pixbuf_get_height (pixbuf);
-        while (h--) {
-                w = gdk_pixbuf_get_width (pixbuf);
-                p = pixels;
-                while (w--) {
-                        p[0] = r;
-                        p[1] = g;
-                        p[2] = b;
-                        p += 4;
-                }
-                pixels += gdk_pixbuf_get_rowstride (pixbuf);
-        }
-}
-
-static void
-style_updated (GtkWidget *widget)
-{
-        g_autoptr(GdkPixbuf) pixbuf = NULL;
-        GtkStyleContext *context;
-        GdkRGBA color;
-        guint r, g, b;
-        guint32 pixel;
-        guint32 old_pixel;
-
-        context = gtk_widget_get_style_context (widget);
-        gtk_style_context_get_color (context, gtk_style_context_get_state (context), &color);
-
-        r = 255 * color.red;
-        g = 255 * color.green;
-        b = 255 * color.blue;
-
-        pixel = (r << 24) | (g  << 16) | (b << 8);
-        old_pixel = GPOINTER_TO_UINT (g_object_get_data (G_OBJECT (widget), "pixel"));
-
-        if (old_pixel == pixel)
-                return;
-
-        g_object_set_data (G_OBJECT (widget), "pixel", GUINT_TO_POINTER (pixel));
-
-        pixbuf = g_object_ref (gtk_image_get_pixbuf (GTK_IMAGE (widget)));
-        pixbuf_fill_rgb (pixbuf, r, g, b);
-        gtk_image_set_from_pixbuf (GTK_IMAGE (widget), pixbuf);
-}
-
 static void
 add_built_logo (GtkAboutDialog *about)
 {
@@ -181,6 +140,9 @@ add_built_logo (GtkAboutDialog *about)
         GtkWidget *button;
         GtkWidget *image;
         g_autoptr(GdkPixbuf) pixbuf = NULL;
+        g_autoptr(GtkCssProvider) provider = NULL;
+        g_autofree char *css = NULL;
+        const char *path;
 
         content = gtk_dialog_get_content_area (GTK_DIALOG (about));
         box = find_child_with_name (content, "box");
@@ -199,10 +161,20 @@ add_built_logo (GtkAboutDialog *about)
         gtk_button_set_relief (GTK_BUTTON (button), GTK_RELIEF_NONE);
         gtk_widget_set_tooltip_text (button, _("Learn more about Builder"));
         gtk_widget_show (button);
-        image = gtk_image_new ();
-        pixbuf = gdk_pixbuf_new_from_resource ("/org/gnome/Recipes/built-with-builder.png", NULL);
-        gtk_image_set_from_pixbuf (GTK_IMAGE (image), pixbuf);
-        g_signal_connect (image, "style-updated", G_CALLBACK (style_updated), NULL);
+
+        image = g_object_new (g_type_from_name ("GtkIcon"), NULL);
+        path = "/org/gnome/Recipes/built-with-builder-symbolic.symbolic.png";
+        pixbuf = gdk_pixbuf_new_from_resource (path, NULL);
+        css = g_strdup_printf (".built-with-builder {\n"
+              "  -gtk-icon-source: -gtk-recolor(url('resource://%s'));\n"
+              "  min-width: %dpx;\n"
+              "  min-height: %dpx;\n"
+              "}", path, gdk_pixbuf_get_width (pixbuf), gdk_pixbuf_get_height (pixbuf));
+        gtk_style_context_add_class (gtk_widget_get_style_context (image), "built-with-builder");
+        provider = gtk_css_provider_new ();
+        gtk_style_context_add_provider_for_screen (gdk_screen_get_default (), GTK_STYLE_PROVIDER (provider), 900);
+        gtk_css_provider_load_from_data (provider, css, -1, NULL);
+
         gtk_widget_show (image);
         gtk_container_add (GTK_CONTAINER (button), image);
 
@@ -697,6 +669,10 @@ about_activated (GSimpleAction *action,
                 "Frederik Fyksen",
                 "Matthias Clasen",
                 "Allan Day",
+                "Erusan",
+                "Link Dupont",
+                "Tuomas Kuosmanen",
+                "Matthew Leeds",
                 NULL
         };
 
@@ -813,7 +789,7 @@ search_activated (GSimpleAction *action,
 
 static GActionEntry app_entries[] =
 {
-        { "preferences", preferences_activated, NULL, NULL, NULL },
+        { "chef-information", chef_information_activated, NULL, NULL, NULL },
         { "about", about_activated, NULL, NULL, NULL },
         { "import", import_activated, NULL, NULL, NULL },
         { "details", details_activated, "(ss)", NULL, NULL },
