@@ -227,7 +227,6 @@ export_one_recipe (GrRecipeExporter  *exporter,
 
                 paths[j] = g_strdup (basename);
                 angles[j] = ri->angle;
-                dark[j] = ri->dark_text;
 
                 j++;
         }
@@ -542,21 +541,6 @@ add_recipe_row (GrRecipeExporter *exporter,
 
 }
 
-static void
-all_headers (GtkListBoxRow *row,
-             GtkListBoxRow *before,
-             gpointer       user_data)
-{
-        GtkWidget *header;
-
-        header = gtk_list_box_row_get_header (row);
-        if (header)
-                return;
-
-        header = gtk_separator_new (GTK_ORIENTATION_HORIZONTAL);
-        gtk_list_box_row_set_header (row, header);
-}
-
 static int
 sort_recipe_row (GtkListBoxRow *row1,
                  GtkListBoxRow *row2,
