@@ -118,9 +118,9 @@ load_recipes (GrRecipeStore *self,
                 g_autofree char *image_path = NULL;
                 g_auto(GStrv) paths = NULL;
                 g_autofree int *angles = NULL;
-                g_autofree gboolean *dark = NULL;
                 int serves;
                 int spiciness;
+                int default_image = 0;
                 GrDiets diets;
                 g_autoptr(GArray) images = NULL;
                 GrRotatedImage ri;
@@ -249,7 +249,7 @@ load_recipes (GrRecipeStore *self,
                         g_warning ("Failed to load recipe %s: Images and Angles length mismatch", groups[i]);
                         continue;
                 }
-                dark = g_key_file_get_boolean_list (keyfile, groups[i], "DarkText", &length3, &error);
+                default_image = g_key_file_get_integer (keyfile, groups[i], "DefaultImage", &error);
                 if (error) {
                         if (!g_error_matches (error, G_KEY_FILE_ERROR, G_KEY_FILE_ERROR_KEY_NOT_FOUND)) {
                                 g_warning ("Failed to load recipe %s: %s", groups[i], error->message);
@@ -257,10 +257,6 @@ load_recipes (GrRecipeStore *self,
                         }
                         g_clear_error (&error);
                 }
-                if (length2 != length3) {
-                        g_warning ("Failed to load recipe %s: Images and DarkText length mismatch", groups[i]);
-                        continue;
-                }
                 serves = g_key_file_get_integer (keyfile, groups[i], "Serves", &error);
                 if (error) {
                         if (!g_error_matches (error, G_KEY_FILE_ERROR, G_KEY_FILE_ERROR_KEY_NOT_FOUND)) {
@@ -306,14 +302,12 @@ load_recipes (GrRecipeStore *self,
                         for (j = 0; paths[j]; j++) {
                                 ri.path = g_strdup (paths[j]);
                                 ri.angle = angles[j];
-                                ri.dark_text = dark[j];
                                 g_array_append_val (images, ri);
                         }
                 }
                 else if (image_path) {
                         ri.path = g_strdup (image_path);
                         ri.angle = 0;
-                        ri.dark_text = FALSE;
                         g_array_append_val (images, ri);
                 }
 
@@ -380,6 +374,7 @@ load_recipes (GrRecipeStore *self,
                                               "spiciness", spiciness,
                                               "diets", diets,
                                               "images", images,
+                                              "default-image", default_image,
                                               "readonly", readonly,
                                               NULL);
                 }
@@ -401,6 +396,7 @@ load_recipes (GrRecipeStore *self,
                                                "spiciness", spiciness,
                                                "diets", diets,
                                                "images", images,
+                                               "default-image", default_image,
                                                "ctime", ctime,
                                                "mtime", mtime,
                                                "readonly", readonly,
@@ -448,9 +444,9 @@ save_recipes (GrRecipeStore *self)
                 GrDiets diets;
                 g_auto(GStrv) paths = NULL;
                 g_autofree int *angles = NULL;
-                g_autofree gboolean *dark = NULL;
                 GDateTime *ctime;
                 GDateTime *mtime;
+                int default_image = 0;
                 int i;
                 gboolean readonly;
 
@@ -470,6 +466,7 @@ save_recipes (GrRecipeStore *self)
                 notes = gr_recipe_get_notes (recipe);
                 ctime = gr_recipe_get_ctime (recipe);
                 mtime = gr_recipe_get_mtime (recipe);
+                default_image = gr_recipe_get_default_image (recipe);
                 readonly = gr_recipe_is_readonly (recipe);
 
                 g_object_get (recipe, "images", &images, NULL);
@@ -477,7 +474,6 @@ save_recipes (GrRecipeStore *self)
                 tmp = get_user_data_dir ();
                 paths = g_new0 (char *, images->len + 1);
                 angles = g_new0 (int, images->len + 1);
-                dark = g_new0 (gboolean, images->len + 1);
                 for (i = 0; i < images->len; i++) {
                         GrRotatedImage *ri = &g_array_index (images, GrRotatedImage, i);
                         if (g_str_has_prefix (ri->path, tmp))
@@ -485,7 +481,6 @@ save_recipes (GrRecipeStore *self)
                         else
                                 paths[i] = g_strdup (ri->path);
                         angles[i] = ri->angle;
-                        dark[i] = ri->dark_text;
                 }
 
                 // For readonly recipes, we just store notes
@@ -508,9 +503,9 @@ save_recipes (GrRecipeStore *self)
                 g_key_file_set_integer (keyfile, key, "Serves", serves);
                 g_key_file_set_integer (keyfile, key, "Spiciness", spiciness);
                 g_key_file_set_integer (keyfile, key, "Diets", diets);
+                g_key_file_set_integer (keyfile, key, "DefaultImage", default_image);
                 g_key_file_set_string_list (keyfile, key, "Images", (const char * const *)paths, images->len);
                 g_key_file_set_integer_list (keyfile, key, "Angles", angles, images->len);
-                g_key_file_set_integer_list (keyfile, key, "DarkText", dark, images->len);
                 if (ctime) {
                         g_autofree char *created = date_time_to_string (ctime);
                         g_key_file_set_string (keyfile, key, "Created", created);
@@ -1282,6 +1277,45 @@ gr_recipe_store_add_chef (GrRecipeStore  *self,
         return TRUE;
 }
 
+gboolean
+gr_recipe_store_update_chef (GrRecipeStore  *self,
+                             GrChef         *chef,
+                             const char     *old_id,
+                             GError        **error)
+{
+        const char *id;
+        GrChef *old;
+
+        g_object_ref (chef);
+
+        id = gr_chef_get_id (chef);
+
+        if (id == NULL || id[0] == '\0') {
+                g_set_error (error, G_IO_ERROR, G_IO_ERROR_FAILED,
+                             _("You need to provide an ID for the chef"));
+                return FALSE;
+        }
+        if (strcmp (id, old_id) != 0 &&
+            g_hash_table_contains (self->chefs, id)) {
+                g_set_error (error, G_IO_ERROR, G_IO_ERROR_FAILED,
+                             _("A chef with this ID already exists"));
+                return FALSE;
+        }
+
+        old = g_hash_table_lookup (self->chefs, old_id);
+        g_assert (chef == old);
+
+        g_hash_table_remove (self->chefs, old_id);
+        g_hash_table_insert (self->chefs, g_strdup (id), g_object_ref (chef));
+
+        g_signal_emit (self, chefs_changed_signal, 0);
+        save_chefs (self);
+
+        g_object_unref (chef);
+
+        return TRUE;
+}
+
 gboolean
 gr_recipe_store_update_user (GrRecipeStore  *self,
                              GrChef         *chef,
