@@ -528,21 +528,6 @@ print_list (GrShoppingPage *page)
         g_list_free_full (items, item_free);
 }
 
-static void
-all_headers (GtkListBoxRow *row,
-             GtkListBoxRow *before,
-             gpointer       user_data)
-{
-        GtkWidget *header;
-
-        header = gtk_list_box_row_get_header (row);
-        if (header)
-                return;
-
-        header = gtk_separator_new (GTK_ORIENTATION_HORIZONTAL);
-        gtk_list_box_row_set_header (row, header);
-}
-
 static void
 gr_shopping_page_init (GrShoppingPage *page)
 {
