@@ -29,10 +29,12 @@
 #include "gr-edit-page.h"
 #include "gr-list-page.h"
 #include "gr-cuisine-page.h"
+#include "gr-cooking-page.h"
 #include "gr-search-page.h"
 #include "gr-shopping-page.h"
 #include "gr-recipes-page.h"
 #include "gr-cuisines-page.h"
+#include "gr-image-page.h"
 #include "gr-query-editor.h"
 #include "gr-recipe-importer.h"
 
@@ -58,14 +60,19 @@ struct _GrWindow
         GtkWidget *search_page;
         GtkWidget *cuisines_page;
         GtkWidget *cuisine_page;
+        GtkWidget *image_page;
+        GtkWidget *cooking_page;
         GtkWidget *undo_revealer;
         GtkWidget *undo_label;
         GrRecipe  *undo_recipe;
         guint undo_timeout_id;
+        GtkWidget *shopping_added_revealer;
+        guint shopping_timeout_id;
 
         GrRecipeImporter *importer;
 
         GQueue *back_entry_stack;
+        gboolean is_fullscreen;
 };
 
 G_DEFINE_TYPE (GrWindow, gr_window, GTK_TYPE_APPLICATION_WINDOW)
@@ -99,7 +106,9 @@ save_back_entry (GrWindow *window)
         const char *page;
 
         page = gtk_stack_get_visible_child_name (GTK_STACK (window->main_stack));
-        if (strcmp (page, "details") == 0 || strcmp (page, "edit") == 0)
+        if (strcmp (page, "details") == 0 ||
+            strcmp (page, "cooking") == 0 ||
+            strcmp (page, "edit") == 0)
                 return;
 
         entry = g_new (BackEntry, 1);
@@ -124,6 +133,8 @@ go_back (GrWindow *window)
 
         entry = g_queue_pop_head (window->back_entry_stack);
 
+        gr_window_set_fullscreen (window, FALSE);
+
         gtk_search_bar_set_search_mode (GTK_SEARCH_BAR (window->search_bar), FALSE);
 
         gtk_header_bar_set_title (GTK_HEADER_BAR (window->header), entry->header_title);
@@ -173,6 +184,17 @@ add_recipe (GrWindow *window)
         }
 }
 
+void
+gr_window_set_fullscreen (GrWindow *window,
+                          gboolean  fullscreen)
+{
+        if (fullscreen)
+                gtk_window_fullscreen (GTK_WINDOW (window));
+        else
+                gtk_window_unfullscreen (GTK_WINDOW (window));
+        window->is_fullscreen = fullscreen;
+}
+
 static void
 stop_search (GrWindow *window)
 {
@@ -260,33 +282,15 @@ search_changed (GrWindow *window)
         gr_search_page_update_search (GR_SEARCH_PAGE (window->search_page), terms);
 }
 
-static void
-update_cooking_button (GrWindow *window,
-                       gboolean  cooking)
-{
-        GtkStyleContext *context;
-
-        context = gtk_widget_get_style_context (window->cooking_button);
-
-        if (cooking) {
-                gtk_button_set_label (GTK_BUTTON (window->cooking_button), _("Stop cooking"));
-                gtk_style_context_remove_class (context, "suggested-action");
-        }
-        else {
-                gtk_button_set_label (GTK_BUTTON (window->cooking_button), _("Start cooking"));
-                gtk_style_context_add_class (context, "suggested-action");
-        }
-}
-
 static void
 start_or_stop_cooking (GrWindow *window)
 {
-        gboolean cooking;
+        GrRecipe *recipe;
 
-        cooking = gr_details_page_is_cooking (GR_DETAILS_PAGE (window->details_page));
-        cooking = !cooking;
-        gr_details_page_set_cooking (GR_DETAILS_PAGE (window->details_page), cooking);
-        update_cooking_button (window, cooking);
+        recipe = gr_details_page_get_recipe (GR_DETAILS_PAGE (window->details_page));
+        gr_cooking_page_set_recipe (GR_COOKING_PAGE (window->cooking_page), recipe);
+        gtk_stack_set_visible_child_name (GTK_STACK (window->main_stack), "cooking");
+        gr_cooking_page_start_cooking (GR_COOKING_PAGE (window->cooking_page));
 }
 
 static gboolean
@@ -295,10 +299,21 @@ window_keypress_handler (GtkWidget *widget,
                          gpointer   data)
 {
         GrWindow *window = GR_WINDOW (widget);
+        GdkEventKey *e = (GdkEventKey *) event;
         const char *visible;
 
         visible = gtk_stack_get_visible_child_name (GTK_STACK (window->main_stack));
 
+        if (strcmp (visible, "image") == 0) {
+                if (e->keyval == GDK_KEY_Escape) {
+                        gr_window_show_image (window, NULL, -1);
+                        return GDK_EVENT_STOP;
+                }
+        }
+
+        if (strcmp (visible, "cooking") == 0)
+                return gr_cooking_page_handle_event (GR_COOKING_PAGE (window->cooking_page), event);
+
         if (strcmp (visible, "recipes") != 0 &&
             strcmp (visible, "cuisines") != 0 &&
             strcmp (visible, "search") != 0)
@@ -306,7 +321,6 @@ window_keypress_handler (GtkWidget *widget,
 
         /* handle ctrl+f shortcut */
         if (event->type == GDK_KEY_PRESS) {
-                GdkEventKey *e = (GdkEventKey *) event;
                 if ((e->state & GDK_CONTROL_MASK) > 0 && e->keyval == GDK_KEY_f) {
                         if (!gtk_search_bar_get_search_mode (GTK_SEARCH_BAR (window->search_bar))) {
                                 gtk_search_bar_set_search_mode (GTK_SEARCH_BAR (window->search_bar), TRUE);
@@ -320,6 +334,22 @@ window_keypress_handler (GtkWidget *widget,
         return gr_query_editor_handle_event (GR_QUERY_EDITOR (window->search_bar), event);
 }
 
+static gboolean
+window_buttonpress_handler (GtkWidget *widget,
+                            GdkEvent  *event,
+                            gpointer   data)
+{
+        GrWindow *window = GR_WINDOW (widget);
+        const char *visible;
+
+        visible = gtk_stack_get_visible_child_name (GTK_STACK (window->main_stack));
+
+        if (strcmp (visible, "cooking") == 0)
+          return gr_cooking_page_handle_event (GR_COOKING_PAGE (window->cooking_page), event);
+
+        return GDK_EVENT_PROPAGATE;
+}
+
 static void
 window_mapped_handler (GtkWidget *widget)
 {
@@ -359,6 +389,10 @@ gr_window_finalize (GObject *object)
                 g_source_remove (self->undo_timeout_id);
                 self->undo_timeout_id = 0;
         }
+        if (self->shopping_timeout_id) {
+                g_source_remove (self->shopping_timeout_id);
+                self->shopping_timeout_id = 0;
+        }
 
         G_OBJECT_CLASS (gr_window_parent_class)->finalize (object);
 }
@@ -416,6 +450,41 @@ gr_window_offer_undelete (GrWindow *window,
         window->undo_timeout_id = g_timeout_add_seconds (10, undo_timeout, window);
 }
 
+static void
+close_shopping_added (GrWindow *window)
+{
+        if (window->shopping_timeout_id) {
+                g_source_remove (window->shopping_timeout_id);
+                window->shopping_timeout_id = 0;
+        }
+
+        gtk_revealer_set_reveal_child (GTK_REVEALER (window->shopping_added_revealer), FALSE);
+}
+
+static gboolean
+shopping_timeout (gpointer data)
+{
+        GrWindow *window = data;
+
+        close_shopping_added (window);
+
+        return G_SOURCE_REMOVE;
+}
+
+static void
+do_shopping_list (GrWindow *window)
+{
+        close_shopping_added (window);
+        gr_window_show_shopping (window);
+}
+
+void
+gr_window_offer_shopping (GrWindow *window)
+{
+        gtk_revealer_set_reveal_child (GTK_REVEALER (window->shopping_added_revealer), TRUE);
+        window->shopping_timeout_id = g_timeout_add_seconds (10, shopping_timeout, window);
+}
+
 static void
 shopping_title_changed (GrWindow *window)
 {
@@ -458,8 +527,11 @@ gr_window_class_init (GrWindowClass *klass)
         gtk_widget_class_bind_template_child (widget_class, GrWindow, search_page);
         gtk_widget_class_bind_template_child (widget_class, GrWindow, cuisines_page);
         gtk_widget_class_bind_template_child (widget_class, GrWindow, cuisine_page);
+        gtk_widget_class_bind_template_child (widget_class, GrWindow, image_page);
+        gtk_widget_class_bind_template_child (widget_class, GrWindow, cooking_page);
         gtk_widget_class_bind_template_child (widget_class, GrWindow, undo_revealer);
         gtk_widget_class_bind_template_child (widget_class, GrWindow, undo_label);
+        gtk_widget_class_bind_template_child (widget_class, GrWindow, shopping_added_revealer);
 
         gtk_widget_class_bind_template_callback (widget_class, new_recipe);
         gtk_widget_class_bind_template_callback (widget_class, go_back);
@@ -471,9 +543,12 @@ gr_window_class_init (GrWindowClass *klass)
         gtk_widget_class_bind_template_callback (widget_class, stop_search);
         gtk_widget_class_bind_template_callback (widget_class, search_mode_changed);
         gtk_widget_class_bind_template_callback (widget_class, window_keypress_handler);
+        gtk_widget_class_bind_template_callback (widget_class, window_buttonpress_handler);
         gtk_widget_class_bind_template_callback (widget_class, window_mapped_handler);
         gtk_widget_class_bind_template_callback (widget_class, do_undo);
         gtk_widget_class_bind_template_callback (widget_class, close_undo);
+        gtk_widget_class_bind_template_callback (widget_class, do_shopping_list);
+        gtk_widget_class_bind_template_callback (widget_class, close_shopping_added);
         gtk_widget_class_bind_template_callback (widget_class, shopping_title_changed);
 }
 
@@ -507,7 +582,6 @@ gr_window_show_recipe (GrWindow *window,
 
         gr_details_page_set_recipe (GR_DETAILS_PAGE (window->details_page), recipe);
 
-        update_cooking_button (window, gr_details_page_is_cooking (GR_DETAILS_PAGE (window->details_page)));
         g_signal_handlers_block_by_func (window->search_bar, search_mode_changed, window);
         gtk_search_bar_set_search_mode (GTK_SEARCH_BAR (window->search_bar), FALSE);
         g_signal_handlers_unblock_by_func (window->search_bar, search_mode_changed, window);
@@ -671,7 +745,7 @@ gr_window_show_shopping (GrWindow *window)
 {
         save_back_entry (window);
 
-        gtk_header_bar_set_title (GTK_HEADER_BAR (window->header), _("Cook it later"));
+        gtk_header_bar_set_title (GTK_HEADER_BAR (window->header), _("Buy ingredients"));
 
         gtk_stack_set_visible_child_name (GTK_STACK (window->header_start_stack), "back");
         gtk_stack_set_visible_child_name (GTK_STACK (window->header_title_stack), "title");
@@ -756,3 +830,20 @@ gr_window_show_search_by_ingredients (GrWindow   *window,
 
         gr_query_editor_set_terms (GR_QUERY_EDITOR (window->search_bar), (const char **)terms);
 }
+
+void
+gr_window_show_image (GrWindow *window,
+                      GArray   *images,
+                      int       index)
+{
+        if (images) {
+                gr_image_page_set_images (GR_IMAGE_PAGE (window->image_page), images);
+                gr_image_page_show_image (GR_IMAGE_PAGE (window->image_page), index);
+                gtk_stack_set_visible_child_name (GTK_STACK (window->main_stack), "image");
+                gr_window_set_fullscreen (window, TRUE);
+        }
+        else {
+                gr_window_set_fullscreen (window, FALSE);
+                gtk_stack_set_visible_child_name (GTK_STACK (window->main_stack), "details");
+        }
+}
