@@ -62,5 +62,13 @@ void            gr_window_show_search_by_ingredients (GrWindow   *window,
                                                       const char *ingredient);
 void            gr_window_offer_undelete             (GrWindow   *window,
                                                       GrRecipe   *recipe);
+void            gr_window_set_fullscreen             (GrWindow   *window,
+                                                      gboolean    fullscreen);
+
+void            gr_window_show_image                 (GrWindow   *window,
+                                                      GArray     *images,
+                                                      int         index);
+
+void            gr_window_offer_shopping             (GrWindow   *window);
 
 G_END_DECLS
